@@ -1,5 +1,5 @@
 #for two exponential term
-smacro.two.out<-function(PKtime,A,alpha,B,beta,defun,par1,par2,par3,par4,i,type)
+smacro.two.out<-function(PKtime,A,alpha,B,beta,defun,par1,par2,par3,par4,i,type,MD=FALSE)
 {
   time<-PKtime$time
   defun<- A*exp(-alpha*time)+B*exp(-beta*time)
@@ -10,7 +10,7 @@ smacro.two.out<-function(PKtime,A,alpha,B,beta,defun,par1,par2,par3,par4,i,type)
   cat(" Model: two-exponential term model          \n") 
   cat(" Error Type:", type,"                       \n\n")
   sim<-matrix(c(A,alpha,B,beta,par1,par2,par3,par4),4,2)
-  dimnames(sim)<-list(c("A","alpha","B","beta"),c("Value","Selected"))
+  dimnames(sim)<-list(c("A","alpha","B","beta"),c("Simulated Values","Input Values"))
   show(sim)
   cat("*******************************************\n\n")
   PKindex<-data.frame(i,output)
@@ -18,6 +18,6 @@ smacro.two.out<-function(PKtime,A,alpha,B,beta,defun,par1,par2,par3,par4,i,type)
   show(PKindex)
   x<-PKindex[,2]
   y<-PKindex[,3]
-  plotting.sim(i,x,y)
+  plotting.sim(i,x,y,MD)
   return(PKindex) 
 }
